Imports System.Data.Entity
Imports System.Linq
Imports DataAccess
Imports DomainClasses
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports System.Data

Namespace IntegrationTests
  <TestClass>
  Public Class UnitTest1
	Public Sub New()
	  Database.SetInitializer(New MyInitializer)
	End Sub

	<TestMethod>
	Public Sub CanSetNavPropertyInNewEntityAndSaveInContext()
	  Using context = New ScreencastContext
		Dim dataTopic = context.Topics.FirstOrDefault(Function(t) t.Name.Contains("Data"))
		Dim originalTopicId = dataTopic.Id
		context.Screencasts.Add(New Screencast With {.Title = "EF101", .Description = "Entity Framework 101", .Topic = dataTopic})
		context.SaveChanges()
	  End Using
	End Sub

	<TestMethod>
	Public Sub SetNavPropertyInNewEntityAndSaveDisconnectedCreatesNewTopicAndId()
	  Dim screencast As Screencast
	  Dim originalTopicId As Integer
	  Using context = New ScreencastContext
		Dim dataTopic = context.Topics.FirstOrDefault(Function(t) t.Name.Contains("Data"))
		screencast = New Screencast With {.Description = "Entity Framework 102", .Topic = dataTopic}
	  End Using
	  Using context = New ScreencastContext
		originalTopicId = screencast.Topic.Id
		context.Screencasts.Add(screencast)
		context.SaveChanges()
		Assert.IsTrue(screencast.TopicId <> originalTopicId)
	  End Using
	End Sub

	<TestMethod>
	Public Sub CanSetFkInNewEntityAndSaveDisconnected()
	  Dim screencast As Screencast
	  Dim originalTopicId As Integer
	  Using context = New ScreencastContext
		Dim dataTopic = context.Topics.FirstOrDefault(Function(t) t.Name.Contains("Data"))
		originalTopicId = dataTopic.Id
		screencast = New Screencast With {.Description = "Entity Framework 102", .TopicId = originalTopicId}
	  End Using
	  Using context = New ScreencastContext
		context.Screencasts.Add(screencast)
		context.SaveChanges()
		Assert.IsTrue(screencast.TopicId = originalTopicId)
	  End Using
	End Sub

	<TestMethod>
	Public Sub AuthorIdRemainsSameAfterAddAuthorsToScreencastAuthorsAndSaveConnected()
	  Dim origAuthorIdA As Integer
	  Dim origAuthorIdB As Integer
	  Dim twoAuthors As List(Of Author)
	  Using context = New ScreencastContext
		twoAuthors = context.Authors.Take(2).ToList
		origAuthorIdA = twoAuthors(0).Id
		origAuthorIdB = twoAuthors(1).Id
		Dim cast = New Screencast With {.Title = "EF 201", .Description = "Entity Framework 201", .TopicId = 1}
                twoAuthors.ForEach(Sub(a) cast.Authors.Add(a))

		context.Screencasts.Add(cast)
		context.SaveChanges()
		Assert.AreEqual(origAuthorIdA, twoAuthors(0).Id)
		Assert.AreEqual(origAuthorIdB, twoAuthors(1).Id)
	  End Using
	End Sub

	<TestMethod>
	Public Sub AuthorIdIsNewAfterAddAuthorsToNewScreencastAuthorsAndSaveDisconnected()

	  Dim origAuthorIdA As Integer
	  Dim origAuthorIdB As Integer
	  Dim twoAuthors As List(Of Author)
	  Using context = New ScreencastContext
		twoAuthors = context.Authors.Take(2).ToList
	  End Using
	  origAuthorIdA = twoAuthors(0).Id
	  origAuthorIdB = twoAuthors(1).Id
	  Dim cast = New Screencast With {.Title = "EF 201", .Description = "Entity Framework 201", .TopicId = 1}
        
            twoAuthors.ForEach(Sub(a) cast.Authors.Add(a))
           
	  Using context = New ScreencastContext
		context.Screencasts.Add(cast)
		context.SaveChanges()
		Assert.AreNotEqual(origAuthorIdA, twoAuthors(0).Id)
		Assert.AreNotEqual(origAuthorIdB, twoAuthors(1).Id)
	  End Using
	End Sub

	<TestMethod>
	Public Sub AuthorIdRemainsSameAfterAddAuthorsToNewScreencastAuthorsAndSaveDisconnectedWithStateFixup()
	  'scenario: app allows you to create screencasts from existing or new authors

	  Dim origAuthorIdA As Integer
	  Dim origAuthorIdB As Integer
	  Dim twoAuthors As List(Of Author)
	  Using context = New ScreencastContext
		twoAuthors = context.Authors.Take(2).ToList
	  End Using
	  origAuthorIdA = twoAuthors(0).Id
            origAuthorIdB = twoAuthors(1).Id
            Dim cast = New Screencast With {.Title = "EF 201", .Description = "Entity Framework 201", .TopicId = 1}
            twoAuthors.ForEach(Sub(a) cast.Authors.Add(a))
	  Using context = New ScreencastContext
		context.Screencasts.Add(cast)
                'explicitly fix state

		cast.Authors.Where(Function(a) a.Id > 0).ToList.ForEach(Function(a) context.Entry(a).State = EntityState.Unchanged)

		context.SaveChanges()
		Assert.AreEqual(origAuthorIdA, twoAuthors(0).Id)
		Assert.AreEqual(origAuthorIdB, twoAuthors(1).Id)
	  End Using
	End Sub

	<TestMethod>
	Public Sub AuthorIdRemainsSameAfterAddAuthorsToNewScreencastAuthorsAndSaveDisconnectedWithAttachAndStateFixup()
	  'scenario: app only allows you to create screencasts from EXISTING authors!

	  Dim origAuthorIdA As Integer
	  Dim origAuthorIdB As Integer
	  Dim twoAuthors As List(Of Author)
	  Using context = New ScreencastContext
		twoAuthors = context.Authors.Take(2).ToList
	  End Using
	  origAuthorIdA = twoAuthors(0).Id
	  origAuthorIdB = twoAuthors(1).Id
	  Dim cast = New Screencast With {.Title = "EF 201", .Description = "Entity Framework 201", .TopicId = 1}

            twoAuthors.ForEach(Sub(a) cast.Authors.Add(a))

	  Using context = New ScreencastContext
		context.Screencasts.Attach(cast)
		'explicitly fix state
		context.Entry(cast).State = EntityState.Added
		cast.Authors.ForEach(Function(a) context.Entry(a).State = EntityState.Unchanged)

		context.SaveChanges()
		Assert.AreEqual(origAuthorIdA, twoAuthors(0).Id)
		Assert.AreEqual(origAuthorIdB, twoAuthors(1).Id)
	  End Using
	End Sub

	<TestMethod>
	Public Sub MoveExistingSectionsToNewScreecastSetsFkToNewScreencastId()
	  Dim sc = New Screencast With {.Title = "EFLong", .Description = "Really Long Course", .TopicId = 3}
	  Dim section = New ScreenCastSection With {.Name = "Part 0"}
	  sc.Sections.Add(New ScreenCastSection With {.Name = "Part 1"})
	  sc.Sections.Add(New ScreenCastSection With {.Name = "Part 2"})
	  sc.Sections.Add(New ScreenCastSection With {.Name = "Part 10"})

	  Using context = New ScreencastContext
		context.Screencasts.Add(sc)
		context.SaveChanges()
	  End Using
	  Dim newScreencast = New Screencast With {.Title = "EF Part 2", .Description = "Second Part of Long Course", .TopicId = 3}
	  Dim part10 = sc.Sections.FirstOrDefault(Function(s) s.Name = "Part 10") 'note I'm looking in sc, not a context...and therefore not the database
	  newScreencast.Sections.Add(part10)
	  Dim originalSection10Id = part10.Id
	  Using context = New ScreencastContext
		context.ScreenCastSections.Attach(part10)
		context.Screencasts.Add(newScreencast)
		context.SaveChanges()
	  End Using
	  Assert.AreEqual(part10.Id, originalSection10Id)
	End Sub
  End Class
End Namespace
