Imports System.Data.Entity
Imports System.Data.Entity.Infrastructure
Imports System.Net
Imports System.Net.Http
Imports System.Web.Http
Imports DomainClasses
Imports DataAccess

Namespace MvcApp.Controllers
	Public Class ScreencastController
		Inherits ApiController

		Private db As New ScreencastContext

		' GET api/Screencast
		Public Function GetScreencasts() As IEnumerable(Of Screencast)
			Dim screencasts = db.Screencasts.Include(Function(s) s.Topic)
			Return screencasts.AsEnumerable
		End Function

		' GET api/Screencast/5
		Public Function GetScreencast(ByVal id As Integer) As Screencast
			Dim screencast As Screencast = db.Screencasts.Find(id)
			If screencast Is Nothing Then
				Throw New HttpResponseException(Request.CreateResponse(HttpStatusCode.NotFound))
			End If

			Return screencast
		End Function

		' PUT api/Screencast/5
		Public Function PutScreencast(ByVal id As Integer, ByVal screencast As Screencast) As HttpResponseMessage
			If ModelState.IsValid AndAlso id = screencast.Id Then
				db.Entry(screencast).State = EntityState.Modified

				Try
					db.SaveChanges()
				Catch e1 As DbUpdateConcurrencyException
					Return Request.CreateResponse(HttpStatusCode.NotFound)
				End Try

				Return Request.CreateResponse(HttpStatusCode.OK)
			Else
				Return Request.CreateResponse(HttpStatusCode.BadRequest)
			End If
		End Function

		' POST api/Screencast
		Public Function PostScreencast(ByVal screencast As Screencast) As HttpResponseMessage
			If ModelState.IsValid Then
				db.Screencasts.Add(screencast)
				db.SaveChanges()

				Dim response As HttpResponseMessage = Request.CreateResponse(HttpStatusCode.Created, screencast)
				response.Headers.Location = New Uri(Url.Link("DefaultApi", New With {Key .id = screencast.Id}))
				Return response
			Else
				Return Request.CreateResponse(HttpStatusCode.BadRequest)
			End If
		End Function

		' DELETE api/Screencast/5
		Public Function DeleteScreencast(ByVal id As Integer) As HttpResponseMessage
			Dim screencast As Screencast = db.Screencasts.Find(id)
			If screencast Is Nothing Then
				Return Request.CreateResponse(HttpStatusCode.NotFound)
			End If

			db.Screencasts.Remove(screencast)

			Try
				db.SaveChanges()
			Catch e1 As DbUpdateConcurrencyException
				Return Request.CreateResponse(HttpStatusCode.NotFound)
			End Try

			Return Request.CreateResponse(HttpStatusCode.OK, screencast)
		End Function

		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			db.Dispose()
			MyBase.Dispose(disposing)
		End Sub
	End Class
End Namespace
